@@ -26,6 +26,7 @@
 #include <QJsonArray>
 #include <QJsonParseError>
 
+#include "defer.h"
 #include "global/io/file.h"
 #include "global/io/dir.h"
 
@@ -34,7 +35,6 @@
 #include "internal/converterutils.h"
 
 #include "log.h"
-#include <iostream>
 
 using namespace mu::converter;
 using namespace mu::project;
@@ -47,20 +47,21 @@ static const std::string PNG_SUFFIX = "png";
 static const std::string SVG_SUFFIX = "svg";
 static const std::string MP3_SUFFIX = "mp3";
 
-Ret ConverterController::batchConvert(const muse::io::path_t& batchJobFile, const muse::io::path_t& stylePath, bool forceMode,
-                                      const String& soundProfile, const muse::UriQuery& extensionUri, muse::ProgressPtr progress)
+Ret ConverterController::batchConvert(const muse::io::path_t& batchJobFile, const OpenParams& openParams,
+                                      const String& soundProfile, const muse::UriQuery& extensionUri,
+                                      muse::ProgressPtr progress)
 {
     TRACEFUNC;
 
     if (progress) {
-        progress->started.notify();
+        progress->start();
     }
 
     RetVal<BatchJob> batchJob = parseBatchJob(batchJobFile);
     if (!batchJob.ret) {
         LOGE() << "failed parse batch job file, err: " << batchJob.ret.toString();
         if (progress) {
-            progress->finished.send(ProgressResult(batchJob.ret));
+            progress->finish(ProgressResult(batchJob.ret));
         }
         return batchJob.ret;
     }
@@ -72,10 +73,10 @@ Ret ConverterController::batchConvert(const muse::io::path_t& batchJobFile, cons
     for (const Job& job : batchJob.val) {
         if (progress) {
             ++current;
-            progress->progressChanged.send(current, total, job.in.toStdString());
+            progress->progress(current, total, job.in.toStdString());
         }
 
-        Ret ret = fileConvert(job.in, job.out, stylePath, forceMode, soundProfile, extensionUri, job.transposeOptions);
+        Ret ret = fileConvert(job.in, job.out, openParams, soundProfile, extensionUri, job.transposeOptions, job.pageNum);
         if (!ret) {
             errors.emplace_back(String(u"failed convert, err: %1, in: %2, out: %3")
                                 .arg(String::fromStdString(ret.toString())).arg(job.in.toString()).arg(job.out.toString()));
@@ -90,18 +91,18 @@ Ret ConverterController::batchConvert(const muse::io::path_t& batchJobFile, cons
     }
 
     if (progress) {
-        progress->finished.send(ProgressResult(ret));
+        progress->finish(ProgressResult(ret));
     }
 
     return ret;
 }
 
 Ret ConverterController::fileConvert(const muse::io::path_t& in, const muse::io::path_t& out,
-                                     const muse::io::path_t& stylePath,
-                                     bool forceMode,
+                                     const OpenParams& openParams,
                                      const muse::String& soundProfile,
                                      const muse::UriQuery& extensionUri,
-                                     const std::string& transposeOptionsJson)
+                                     const std::string& transposeOptionsJson,
+                                     const std::optional<size_t>& pageNum)
 {
     std::optional<TransposeOptions> transposeOptions;
 
@@ -114,26 +115,33 @@ Ret ConverterController::fileConvert(const muse::io::path_t& in, const muse::io:
         transposeOptions = transposeOptionsRet.val;
     }
 
-    return fileConvert(in, out, stylePath, forceMode, soundProfile, extensionUri, transposeOptions);
+    return fileConvert(in, out, openParams, soundProfile, extensionUri, transposeOptions, pageNum);
 }
 
 Ret ConverterController::fileConvert(const muse::io::path_t& in, const muse::io::path_t& out,
-                                     const muse::io::path_t& stylePath,
-                                     bool forceMode,
+                                     const OpenParams& openParams,
                                      const String& soundProfile,
                                      const muse::UriQuery& extensionUri,
-                                     const std::optional<notation::TransposeOptions>& transposeOptions)
+                                     const std::optional<notation::TransposeOptions>& transposeOptions,
+                                     const std::optional<size_t>& pageNum)
 {
     TRACEFUNC;
 
     LOGI() << "in: " << in << ", out: " << out;
 
+    std::string suffix = io::suffix(out);
+
+    auto writer = writers()->writer(suffix);
+    if (!writer) {
+        return make_ret(Err::ConvertTypeUnknown);
+    }
+
     auto notationProject = notationCreator()->newProject(iocContext());
     IF_ASSERT_FAILED(notationProject) {
         return make_ret(Err::UnknownError);
     }
 
-    Ret ret = notationProject->load(in, stylePath, forceMode);
+    Ret ret = notationProject->load(in, openParams.stylePath, openParams.forceMode);
     if (!ret) {
         LOGE() << "failed load notation, err: " << ret.toString() << ", path: " << in;
         return make_ret(Err::InFileFailedLoad);
@@ -154,17 +162,14 @@ Ret ConverterController::fileConvert(const muse::io::path_t& in, const muse::io:
 
     globalContext()->setCurrentProject(notationProject);
 
+    DEFER {
+        globalContext()->setCurrentProject(nullptr);
+    };
+
     // Check if this is a part conversion job
     QString baseName = QString::fromStdString(io::completeBasename(out).toStdString());
     if (baseName.contains('*')) {
-        return convertScoreParts(in, out, stylePath, forceMode);
-    }
-
-    std::string suffix = io::suffix(out);
-
-    auto writer = writers()->writer(suffix);
-    if (!writer) {
-        return make_ret(Err::ConvertTypeUnknown);
+        return convertScoreParts(writer, notationProject->masterNotation(), out);
     }
 
     // use a extension for convert
@@ -177,11 +182,20 @@ Ret ConverterController::fileConvert(const muse::io::path_t& in, const muse::io:
     // standart convert
     else {
         if (suffix == engraving::MSCZ || suffix == engraving::MSCX || suffix == engraving::MSCS) {
+            if (pageNum.has_value()) {
+                return notationProject->savePage(out, pageNum.value());
+            }
+
             return notationProject->save(out);
         }
 
-        if (isConvertPageByPage(suffix)) {
-            ret = convertPageByPage(writer, notationProject->masterNotation()->notation(), out);
+        if (pageNum.has_value() || isConvertPageByPage(suffix)) {
+            if (pageNum.has_value()) {
+                ret = convertPage(writer, notationProject->masterNotation()->notation(), pageNum.value(), out);
+            } else {
+                ret = convertPageByPage(writer, notationProject->masterNotation()->notation(), out);
+            }
+
             if (!ret) {
                 LOGE() << "Failed to convert page by page, err: " << ret.toString();
             }
@@ -193,13 +207,10 @@ Ret ConverterController::fileConvert(const muse::io::path_t& in, const muse::io:
         }
     }
 
-    globalContext()->setCurrentProject(nullptr);
-
     return ret;
 }
 
-Ret ConverterController::convertScoreParts(const muse::io::path_t& in, const muse::io::path_t& out, const muse::io::path_t& stylePath,
-                                           bool forceMode)
+Ret ConverterController::convertScoreParts(const muse::io::path_t& in, const muse::io::path_t& out, const OpenParams& openParams)
 {
     TRACEFUNC;
 
@@ -214,23 +225,30 @@ Ret ConverterController::convertScoreParts(const muse::io::path_t& in, const mus
         return make_ret(Err::ConvertTypeUnknown);
     }
 
-    Ret ret = notationProject->load(in, stylePath, forceMode);
+    Ret ret = notationProject->load(in, openParams.stylePath, openParams.forceMode);
     if (!ret) {
         LOGE() << "failed load notation, err: " << ret.toString() << ", path: " << in;
         return make_ret(Err::InFileFailedLoad);
     }
 
+    ret = convertScoreParts(writer, notationProject->masterNotation(), out);
+
+    return ret;
+}
+
+Ret ConverterController::convertScoreParts(INotationWriterPtr writer, IMasterNotationPtr masterNotation, const muse::io::path_t& out)
+{
+    std::string suffix = io::suffix(out);
+
     if (suffix == PDF_SUFFIX) {
-        ret = convertScorePartsToPdf(writer, notationProject->masterNotation(), out);
+        return convertScorePartsToPdf(writer, masterNotation, out);
     } else if (suffix == PNG_SUFFIX) {
-        ret = convertScorePartsToPngs(writer, notationProject->masterNotation(), out);
+        return convertScorePartsToPngs(writer, masterNotation, out);
     } else if (suffix == MP3_SUFFIX) {
-        ret = convertScorePartsToMp3(writer, notationProject->masterNotation(), out);
-    } else {
-        ret = make_ret(Ret::Code::NotSupported);
+        return convertScorePartsToMp3(writer, masterNotation, out);
     }
 
-    return make_ret(Ret::Code::Ok);
+    return make_ret(Ret::Code::NotSupported);
 }
 
 RetVal<ConverterController::BatchJob> ConverterController::parseBatchJob(const muse::io::path_t& batchJobFile) const
@@ -240,31 +258,30 @@ RetVal<ConverterController::BatchJob> ConverterController::parseBatchJob(const m
     RetVal<BatchJob> rv;
     QFile file(batchJobFile.toQString());
     if (!file.open(QIODevice::ReadOnly)) {
-        rv.ret = make_ret(Err::BatchJobFileFailedOpen);
+        rv.ret = make_ret(Err::BatchJobFileFailedOpen, file.errorString().toStdString());
         return rv;
     }
 
-    QByteArray data = file.readAll();
+    const QByteArray data = file.readAll();
+
     QJsonParseError err;
-    QJsonDocument doc = QJsonDocument::fromJson(data, &err);
+    const QJsonDocument doc = QJsonDocument::fromJson(data, &err);
     if (err.error != QJsonParseError::NoError || !doc.isArray()) {
         rv.ret = make_ret(Err::BatchJobFileFailedParse, err.errorString().toStdString());
         return rv;
     }
 
-    QJsonArray arr = doc.array();
+    const QJsonArray arr = doc.array();
 
     auto correctUserInputPath = [](const QString& path) -> QString {
         return io::Dir::fromNativeSeparators(path).toQString();
     };
 
-    for (const QJsonValue& v : arr) {
-        QJsonObject obj = v.toObject();
-
+    for (const auto obj : arr) {
         Job job;
-        job.in = correctUserInputPath(obj["in"].toString());
+        job.in = correctUserInputPath(obj[u"in"].toString());
 
-        QJsonObject transposeOptionsObj = obj["transpose"].toObject();
+        QJsonObject transposeOptionsObj = obj[u"transpose"].toObject();
         if (!transposeOptionsObj.isEmpty()) {
             RetVal<TransposeOptions> transposeOptions = ConverterUtils::parseTransposeOptions(transposeOptionsObj);
             if (!transposeOptions.ret) {
@@ -274,20 +291,25 @@ RetVal<ConverterController::BatchJob> ConverterController::parseBatchJob(const m
             job.transposeOptions = transposeOptions.val;
         }
 
-        QJsonValue outValue = obj["out"];
+        QJsonValue pageVal = obj[u"page"];
+        if (!pageVal.isUndefined()) {
+            job.pageNum = pageVal.toInt() - 1;
+        }
+
+        const QJsonValue outValue = obj[u"out"];
         if (outValue.isString()) {
             job.out = correctUserInputPath(outValue.toString());
             rv.val.push_back(std::move(job));
         } else if (outValue.isArray()) {
-            QJsonArray outArray = outValue.toArray();
-            for (const QJsonValue& outItem : outArray) {
+            const QJsonArray outArray = outValue.toArray();
+            for (const auto outItem : outArray) {
                 Job partJob = job; // Copy the input path
                 if (outItem.isString()) {
                     partJob.out = correctUserInputPath(outItem.toString());
                 } else if (outItem.isArray() && outItem.toArray().size() == 2) {
-                    QJsonArray partOutArray = outItem.toArray();
-                    QString prefix = correctUserInputPath(partOutArray[0].toString());
-                    QString suffix = partOutArray[1].toString();
+                    const QJsonArray partOutArray = outItem.toArray();
+                    const QString prefix = correctUserInputPath(partOutArray[0].toString());
+                    const QString suffix = partOutArray[1].toString();
                     partJob.out = prefix + "*" + suffix; // Use "*" as a placeholder for part names
                 }
                 rv.val.push_back(std::move(partJob));
@@ -327,12 +349,12 @@ Ret ConverterController::convertByExtension(INotationWriterPtr writer, INotation
 
 bool ConverterController::isConvertPageByPage(const std::string& suffix) const
 {
-    QList<std::string> types {
+    static const std::unordered_set<std::string> TYPES {
         PNG_SUFFIX,
-        SVG_SUFFIX
+        SVG_SUFFIX,
     };
 
-    return types.contains(suffix);
+    return muse::contains(TYPES, suffix);
 }
 
 Ret ConverterController::convertPageByPage(INotationWriterPtr writer, INotationPtr notation, const muse::io::path_t& out) const
@@ -344,30 +366,45 @@ Ret ConverterController::convertPageByPage(INotationWriterPtr writer, INotationP
                                                  + io::completeBasename(out) + "-%1."
                                                  + io::suffix(out)).toString().arg(i + 1);
 
-        File file(filePath);
-        if (!file.open(File::WriteOnly)) {
-            return make_ret(Err::OutFileFailedOpen);
-        }
-
-        INotationWriter::Options options = {
-            { INotationWriter::OptionKey::PAGE_NUMBER, Val(static_cast<int>(i)) },
-        };
-
-        file.setMeta("dir_path", out.toStdString());
-        file.setMeta("file_path", filePath.toStdString());
-
-        Ret ret = writer->write(notation, file, options);
+        Ret ret = convertPage(writer, notation, i, filePath, out);
         if (!ret) {
-            LOGE() << "failed write, err: " << ret.toString() << ", path: " << out;
-            return make_ret(Err::OutFileFailedWrite);
+            return ret;
         }
-
-        file.close();
     }
 
     return make_ret(Ret::Code::Ok);
 }
 
+muse::Ret ConverterController::convertPage(INotationWriterPtr writer, INotationPtr notation, const size_t pageNum,
+                                           const muse::io::path_t& filePath, const muse::io::path_t& dirPath) const
+{
+    TRACEFUNC;
+
+    File file(filePath);
+    if (!file.open(File::WriteOnly)) {
+        return make_ret(Err::OutFileFailedOpen);
+    }
+
+    const INotationWriter::Options options {
+        { INotationWriter::OptionKey::PAGE_NUMBER, Val(static_cast<int>(pageNum)) },
+    };
+
+    file.setMeta("file_path", filePath.toStdString());
+
+    if (!dirPath.empty()) {
+        file.setMeta("dir_path", dirPath.toStdString());
+    }
+
+    Ret ret = writer->write(notation, file, options);
+    if (!ret) {
+        return make_ret(Err::OutFileFailedWrite);
+    }
+
+    file.close();
+
+    return make_ok();
+}
+
 Ret ConverterController::convertFullNotation(INotationWriterPtr writer, INotationPtr notation, const muse::io::path_t& out) const
 {
     File file(out);
@@ -392,13 +429,12 @@ Ret ConverterController::convertScorePartsToPdf(INotationWriterPtr writer, IMast
 {
     TRACEFUNC;
 
-    INotationPtrList notations;
-    for (IExcerptNotationPtr e : masterNotation->excerpts()) {
-        notations.push_back(e->notation());
-    }
+    const INotationWriter::Options options {
+        { INotationWriter::OptionKey::UNIT_TYPE, Val(INotationWriter::UnitType::PER_PART) },
+    };
 
-    for (size_t i = 0; i < notations.size(); ++i) {
-        QString partName = notations[i]->name();
+    for (const IExcerptNotationPtr& e : masterNotation->excerpts()) {
+        QString partName = e->notation()->name();
         QString baseName = QString::fromStdString(io::completeBasename(out).toStdString());
         muse::io::path_t partOut = io::dirpath(out) + "/" + baseName.replace("*", partName).toStdString() + ".pdf";
 
@@ -407,11 +443,7 @@ Ret ConverterController::convertScorePartsToPdf(INotationWriterPtr writer, IMast
             return make_ret(Err::OutFileFailedOpen);
         }
 
-        INotationWriter::Options options {
-            { INotationWriter::OptionKey::UNIT_TYPE, Val(INotationWriter::UnitType::PER_PART) },
-        };
-
-        Ret ret = writer->write(notations[i], file, options);
+        Ret ret = writer->write(e->notation(), file, options);
         if (!ret) {
             LOGE() << "failed write, err: " << ret.toString() << ", path: " << partOut;
             return make_ret(Err::OutFileFailedWrite);
@@ -428,18 +460,13 @@ Ret ConverterController::convertScorePartsToPngs(INotationWriterPtr writer, mu::
 {
     TRACEFUNC;
 
-    INotationPtrList notations;
-    for (IExcerptNotationPtr e : masterNotation->excerpts()) {
-        notations.push_back(e->notation());
-    }
-
-    for (size_t i = 0; i < notations.size(); i++) {
-        QString partName = notations[i]->name();
+    for (const IExcerptNotationPtr& e : masterNotation->excerpts()) {
+        QString partName = e->notation()->name();
         QString baseName = QString::fromStdString(io::completeBasename(out).toStdString());
         muse::io::path_t pngFilePath = io::dirpath(out) + "/" + baseName.replace("*", partName).toStdString() + ".png";
-        Ret ret2 = convertPageByPage(writer, notations[i], pngFilePath);
-        if (!ret2) {
-            return ret2;
+        Ret ret = convertPageByPage(writer, e->notation(), pngFilePath);
+        if (!ret) {
+            return ret;
         }
     }
 
@@ -451,13 +478,12 @@ Ret ConverterController::convertScorePartsToMp3(INotationWriterPtr writer, IMast
 {
     TRACEFUNC;
 
-    INotationPtrList notations;
-    for (IExcerptNotationPtr e : masterNotation->excerpts()) {
-        notations.push_back(e->notation());
-    }
+    const INotationWriter::Options options {
+        { INotationWriter::OptionKey::UNIT_TYPE, Val(INotationWriter::UnitType::PER_PART) },
+    };
 
-    for (size_t i = 0; i < notations.size(); ++i) {
-        QString partName = notations[i]->name();
+    for (const IExcerptNotationPtr& e : masterNotation->excerpts()) {
+        QString partName = e->notation()->name();
         QString baseName = QString::fromStdString(io::completeBasename(out).toStdString());
         muse::io::path_t partOut = io::dirpath(out) + "/" + baseName.replace("*", partName).toStdString() + ".mp3";
 
@@ -466,11 +492,8 @@ Ret ConverterController::convertScorePartsToMp3(INotationWriterPtr writer, IMast
             return make_ret(Err::OutFileFailedOpen);
         }
 
-        INotationWriter::Options options {
-            { INotationWriter::OptionKey::UNIT_TYPE, Val(INotationWriter::UnitType::PER_PART) },
-        };
         file.setMeta("file_path", partOut.toStdString());
-        Ret ret = writer->write(notations[i], file, options);
+        Ret ret = writer->write(e->notation(), file, options);
         if (!ret) {
             LOGE() << "failed write, err: " << ret.toString() << ", path: " << partOut;
             return make_ret(Err::OutFileFailedWrite);
@@ -483,44 +506,49 @@ Ret ConverterController::convertScorePartsToMp3(INotationWriterPtr writer, IMast
 }
 
 Ret ConverterController::exportScoreMedia(const muse::io::path_t& in, const muse::io::path_t& out,
-                                          const muse::io::path_t& highlightConfigPath,
-                                          const muse::io::path_t& stylePath, bool forceMode)
+                                          const OpenParams& openParams,
+                                          const muse::io::path_t& highlightConfigPath)
 {
     TRACEFUNC;
 
-    return BackendApi::exportScoreMedia(in, out, highlightConfigPath, stylePath, forceMode);
+    return BackendApi::exportScoreMedia(in, out, highlightConfigPath, openParams.stylePath, openParams.forceMode);
 }
 
-Ret ConverterController::exportScoreMeta(const muse::io::path_t& in, const muse::io::path_t& out, const muse::io::path_t& stylePath,
-                                         bool forceMode)
+Ret ConverterController::exportScoreMeta(const muse::io::path_t& in, const muse::io::path_t& out, const OpenParams& openParams)
 {
     TRACEFUNC;
 
-    return BackendApi::exportScoreMeta(in, out, stylePath, forceMode);
+    return BackendApi::exportScoreMeta(in, out, openParams.stylePath, openParams.forceMode);
 }
 
-Ret ConverterController::exportScoreParts(const muse::io::path_t& in, const muse::io::path_t& out, const muse::io::path_t& stylePath,
-                                          bool forceMode)
+Ret ConverterController::exportScoreParts(const muse::io::path_t& in, const muse::io::path_t& out, const OpenParams& openParams)
 {
     TRACEFUNC;
 
-    return BackendApi::exportScoreParts(in, out, stylePath, forceMode);
+    return BackendApi::exportScoreParts(in, out, openParams.stylePath, openParams.forceMode);
 }
 
-Ret ConverterController::exportScorePartsPdfs(const muse::io::path_t& in, const muse::io::path_t& out, const muse::io::path_t& stylePath,
-                                              bool forceMode)
+Ret ConverterController::exportScorePartsPdfs(const muse::io::path_t& in, const muse::io::path_t& out, const OpenParams& openParams)
 {
     TRACEFUNC;
 
-    return BackendApi::exportScorePartsPdfs(in, out, stylePath, forceMode);
+    return BackendApi::exportScorePartsPdfs(in, out, openParams.stylePath, openParams.forceMode);
 }
 
 Ret ConverterController::exportScoreTranspose(const muse::io::path_t& in, const muse::io::path_t& out, const std::string& optionsJson,
-                                              const muse::io::path_t& stylePath, bool forceMode)
+                                              const OpenParams& openParams)
 {
     TRACEFUNC;
 
-    return BackendApi::exportScoreTranspose(in, out, optionsJson, stylePath, forceMode);
+    return BackendApi::exportScoreTranspose(in, out, optionsJson, openParams.stylePath, openParams.forceMode);
+}
+
+Ret ConverterController::exportScoreElements(const muse::io::path_t& in, const muse::io::path_t& out, const std::string& optionsJson,
+                                             const OpenParams& openParams)
+{
+    TRACEFUNC;
+
+    return BackendApi::exportScoreElements(in, out, optionsJson, openParams.stylePath, openParams.forceMode);
 }
 
 Ret ConverterController::exportScoreVideo(const muse::io::path_t& in, const muse::io::path_t& out)
