@@ -20,8 +20,7 @@
  * along with this program.  If not, see <https://www.gnu.org/licenses/>.
  */
 
-#ifndef __MIDIPATCH_H__
-#define __MIDIPATCH_H__
+#pragma once
 
 #include "types/string.h"
 
@@ -32,9 +31,8 @@ namespace mu::engraving {
 
 struct MidiPatch {
     bool drum;
-    String synti;
+    muse::String synti;
     int bank, prog, sfid;
-    String name;
+    muse::String name;
 };
 }
-#endif
