@@ -133,12 +133,12 @@ void MusicXml_Tests::musicXmlIoTest(const char* file, bool exportLayout)
     setValue(PREF_EXPORT_MUSICXML_EXPORTBREAKS, Val(IMusicXmlConfiguration::MusicXmlExportBreaksType::Manual));
     setValue(PREF_IMPORT_MUSICXML_IMPORTBREAKS, Val(true));
     setValue(PREF_EXPORT_MUSICXML_EXPORTLAYOUT, Val(exportLayout));
-    setValue(PREF_EXPORT_MUSICXML_EXPORTINVISIBLE, Val(true));
+    setValue(PREF_EXPORT_MUSICXML_EXPORTINVISIBLE, Val(false));
     setValue(PREF_IMPORT_MUSICXML_INFERTEXT, Val(true));
 
     String fileName = String::fromUtf8(file);
     MasterScore* score = readScore(XML_IO_DATA_DIR + fileName + u".xml");
-    EXPECT_TRUE(score);
+    ASSERT_TRUE(score);
     fixupScore(score);
     score->doLayout();
     EXPECT_TRUE(saveCompareMusicXmlScore(score, fileName + u".xml", XML_IO_DATA_DIR + fileName + u".xml"));
@@ -162,7 +162,7 @@ void MusicXml_Tests::musicXmlIoTestRef(const char* file)
 
     String fileName = String::fromUtf8(file);
     MasterScore* score = readScore(XML_IO_DATA_DIR + fileName + u".xml");
-    EXPECT_TRUE(score);
+    ASSERT_TRUE(score);
     fixupScore(score);
     score->doLayout();
     EXPECT_TRUE(saveCompareMusicXmlScore(score, fileName + u".xml", XML_IO_DATA_DIR + fileName + u"_ref.xml"));
@@ -186,21 +186,21 @@ void MusicXml_Tests::musicXmlIoTestRefBreaks(const char* file)
 
     String fileName = String::fromUtf8(file);
     MasterScore* score = readScore(XML_IO_DATA_DIR + fileName + u".xml");
-    EXPECT_TRUE(score);
+    ASSERT_TRUE(score);
     fixupScore(score);
     score->doLayout();
 
     setValue(PREF_EXPORT_MUSICXML_EXPORTBREAKS, Val(IMusicXmlConfiguration::MusicXmlExportBreaksType::No));
 
-    EXPECT_TRUE(saveCompareMusicXmlScore(score, fileName + u".xml", XML_IO_DATA_DIR + fileName + u"_no_ref.xml"));
+    EXPECT_TRUE(saveCompareMusicXmlScore(score, fileName + u"_no.xml", XML_IO_DATA_DIR + fileName + u"_no_ref.xml"));
 
     setValue(PREF_EXPORT_MUSICXML_EXPORTBREAKS, Val(IMusicXmlConfiguration::MusicXmlExportBreaksType::Manual));
 
-    EXPECT_TRUE(saveCompareMusicXmlScore(score, fileName + u".xml", XML_IO_DATA_DIR + fileName + u"_manual_ref.xml"));
+    EXPECT_TRUE(saveCompareMusicXmlScore(score, fileName + u"_manual.xml", XML_IO_DATA_DIR + fileName + u"_manual_ref.xml"));
 
     setValue(PREF_EXPORT_MUSICXML_EXPORTBREAKS, Val(IMusicXmlConfiguration::MusicXmlExportBreaksType::All));
 
-    EXPECT_TRUE(saveCompareMusicXmlScore(score, fileName + u".xml", XML_IO_DATA_DIR + fileName + u"_all_ref.xml"));
+    EXPECT_TRUE(saveCompareMusicXmlScore(score, fileName + u"_all.xml", XML_IO_DATA_DIR + fileName + u"_all_ref.xml"));
     delete score;
 }
 
@@ -220,7 +220,7 @@ void MusicXml_Tests::musicXmlMscxExportTestRef(const char* file, bool exportLayo
 
     String fileName = String::fromUtf8(file);
     MasterScore* score = readScore(XML_IO_DATA_DIR + fileName + u".mscx");
-    EXPECT_TRUE(score);
+    ASSERT_TRUE(score);
     fixupScore(score);
     score->doLayout();
 
@@ -245,21 +245,21 @@ void MusicXml_Tests::musicXmlMscxExportTestRefBreaks(const char* file)
 
     String fileName = String::fromUtf8(file);
     MasterScore* score = readScore(XML_IO_DATA_DIR + fileName + u".mscx");
-    EXPECT_TRUE(score);
+    ASSERT_TRUE(score);
     fixupScore(score);
     score->doLayout();
 
     setValue(PREF_EXPORT_MUSICXML_EXPORTBREAKS, Val(IMusicXmlConfiguration::MusicXmlExportBreaksType::No));
 
-    EXPECT_TRUE(saveCompareMusicXmlScore(score, fileName + u".xml", XML_IO_DATA_DIR + fileName + u"_no_ref.xml"));
+    EXPECT_TRUE(saveCompareMusicXmlScore(score, fileName + u"_no.xml", XML_IO_DATA_DIR + fileName + u"_no_ref.xml"));
 
     setValue(PREF_EXPORT_MUSICXML_EXPORTBREAKS, Val(IMusicXmlConfiguration::MusicXmlExportBreaksType::Manual));
 
-    EXPECT_TRUE(saveCompareMusicXmlScore(score, fileName + u".xml", XML_IO_DATA_DIR + fileName + u"_manual_ref.xml"));
+    EXPECT_TRUE(saveCompareMusicXmlScore(score, fileName + u"_manual.xml", XML_IO_DATA_DIR + fileName + u"_manual_ref.xml"));
 
     setValue(PREF_EXPORT_MUSICXML_EXPORTBREAKS, Val(IMusicXmlConfiguration::MusicXmlExportBreaksType::All));
 
-    EXPECT_TRUE(saveCompareMusicXmlScore(score, fileName + u".xml", XML_IO_DATA_DIR + fileName + u"_all_ref.xml"));
+    EXPECT_TRUE(saveCompareMusicXmlScore(score, fileName + u"_all.xml", XML_IO_DATA_DIR + fileName + u"_all_ref.xml"));
     delete score;
 }
 
@@ -273,17 +273,17 @@ void MusicXml_Tests::musicXmlMscxExportTestRefInvisibleElements(const char* file
 
     String fileName = String::fromUtf8(file);
     MasterScore* score = readScore(XML_IO_DATA_DIR + fileName + u".mscx");
-    EXPECT_TRUE(score);
+    ASSERT_TRUE(score);
     fixupScore(score);
     score->doLayout();
 
     setValue(PREF_EXPORT_MUSICXML_EXPORTINVISIBLE, Val(true));
 
-    EXPECT_TRUE(saveCompareMusicXmlScore(score, fileName + u".xml", XML_IO_DATA_DIR + fileName + u"_invisible_ref.xml"));
+    EXPECT_TRUE(saveCompareMusicXmlScore(score, fileName + u"_invisible.xml", XML_IO_DATA_DIR + fileName + u"_invisible_ref.xml"));
 
     setValue(PREF_EXPORT_MUSICXML_EXPORTINVISIBLE, Val(false));
 
-    EXPECT_TRUE(saveCompareMusicXmlScore(score, fileName + u".xml", XML_IO_DATA_DIR + fileName + u"_noinvisible_ref.xml"));
+    EXPECT_TRUE(saveCompareMusicXmlScore(score, fileName + u"_noinvisible.xml", XML_IO_DATA_DIR + fileName + u"_noinvisible_ref.xml"));
 
     delete score;
 }
@@ -304,7 +304,7 @@ void MusicXml_Tests::musicXmlReadTestCompr(const char* file)
 
     String fileName = String::fromUtf8(file);
     MasterScore* score = readScore(XML_IO_DATA_DIR + fileName + u".mxl");
-    EXPECT_TRUE(score);
+    ASSERT_TRUE(score);
     fixupScore(score);
     score->doLayout();
     EXPECT_TRUE(saveCompareMusicXmlScore(score, fileName + u"_mxl_read.xml", XML_IO_DATA_DIR + fileName + u".xml"));
@@ -329,7 +329,7 @@ void MusicXml_Tests::musicXmlReadWriteTestCompr(const char* file)
 
     String fileName = String::fromUtf8(file);
     MasterScore* score = readScore(XML_IO_DATA_DIR + fileName + u".xml");
-    EXPECT_TRUE(score);
+    ASSERT_TRUE(score);
     fixupScore(score);
     score->doLayout();
     // write mxl
@@ -360,7 +360,7 @@ void MusicXml_Tests::musicXmlImportTestRef(const char* file)
 
     String fileName = String::fromUtf8(file);
     MasterScore* score = readScore(XML_IO_DATA_DIR + fileName + u".xml");
-    EXPECT_TRUE(score);
+    ASSERT_TRUE(score);
     fixupScore(score);
     score->doLayout();
     EXPECT_TRUE(ScoreComp::saveCompareScore(score, fileName + u".mscx", XML_IO_DATA_DIR + fileName + u"_ref.mscx"));
@@ -436,16 +436,19 @@ TEST_F(MusicXml_Tests, beams2) {
 TEST_F(MusicXml_Tests, beams3) {
     musicXmlIoTestRef("testBeams3");
 }
+TEST_F(MusicXml_Tests, beams4) {
+    musicXmlIoTest("testBeams4");
+}
 TEST_F(MusicXml_Tests, breaksImplExpl) {
     musicXmlMscxExportTestRefBreaks("testBreaksImplExpl");
 }
 TEST_F(MusicXml_Tests, breaksMMRest) {
     musicXmlMscxExportTestRefBreaks("testBreaksMMRest");
 }
-TEST_F(MusicXml_Tests, DISABLED_breaksManual) { // fail after sync with 3.x
+TEST_F(MusicXml_Tests, breaksManual) {
     musicXmlIoTestRefBreaks("testBreaksManual");
 }
-TEST_F(MusicXml_Tests, DISABLED_breaksPage) { // fail after sync with 3.x
+TEST_F(MusicXml_Tests, breaksPage) {
     musicXmlMscxExportTestRefBreaks("testBreaksPage");
 }
 TEST_F(MusicXml_Tests, breaksSystem) {
@@ -643,6 +646,9 @@ TEST_F(MusicXml_Tests, fractionPlus) {
 TEST_F(MusicXml_Tests, fractionTicks) {
     musicXmlIoTestRef("testFractionTicks");
 }
+TEST_F(MusicXml_Tests, glissandoLines) {
+    musicXmlIoTest("testGlissandoLines");
+}
 TEST_F(MusicXml_Tests, glissFall) {
     musicXmlImportTestRef("testGlissFall");
 }
@@ -676,14 +682,23 @@ TEST_F(MusicXml_Tests, guitarBends) {
     musicXmlMscxExportTestRef("testGuitarBends");
     MScore::useRead302InTestMode = useRead302;
 }
+TEST_F(MusicXml_Tests, hammerPull) {
+    bool useRead302 = MScore::useRead302InTestMode;
+    MScore::useRead302InTestMode = false;
+    musicXmlIoTest("testHammerPull");
+    MScore::useRead302InTestMode = useRead302;
+}
 TEST_F(MusicXml_Tests, harpPedals) {
     musicXmlMscxExportTestRef("testHarpPedals");
 }
 TEST_F(MusicXml_Tests, hairpinDynamics) {
     musicXmlMscxExportTestRef("testHairpinDynamics");
 }
-TEST_F(MusicXml_Tests, handbells) {
-    musicXmlIoTest("testHandbells");
+TEST_F(MusicXml_Tests, handbells1) {
+    musicXmlIoTest("testHandbells1");
+}
+TEST_F(MusicXml_Tests, handbells2) {
+    musicXmlIoTest("testHandbells2");
 }
 TEST_F(MusicXml_Tests, harmony1) {
     musicXmlIoTest("testHarmony1");
@@ -698,8 +713,8 @@ TEST_F(MusicXml_Tests, harmony4) {
     musicXmlIoTest("testHarmony4");
 }
 TEST_F(MusicXml_Tests, harmony5) {
-    musicXmlIoTest("testHarmony5");
-}                                                                      // chordnames without chordrest
+    musicXmlIoTest("testHarmony5"); // chordnames without chordrest
+}
 TEST_F(MusicXml_Tests, harmony6) {
     musicXmlMscxExportTestRef("testHarmony6");
 }
@@ -710,8 +725,8 @@ TEST_F(MusicXml_Tests, harmony8) {
     musicXmlIoTest("testHarmony8");
 }
 TEST_F(MusicXml_Tests, harmony9) {
-    musicXmlIoTest("testHarmony9");
-}                                                                      // chordnames without chordrest
+    musicXmlIoTest("testHarmony9"); // chordnames without chordrest
+}
 TEST_F(MusicXml_Tests, harmonMutes) {
     musicXmlIoTest("testHarmonMutes");
 }
@@ -742,17 +757,19 @@ TEST_F(MusicXml_Tests, incompleteTuplet) {
 TEST_F(MusicXml_Tests, incorrectMidiProgram) {
     musicXmlIoTestRef("testIncorrectMidiProgram");
 }
-
+TEST_F(MusicXml_Tests, invisibleNotations) {
+    musicXmlIoTest("testInvisibleNotations");
+}
 TEST_F(MusicXml_Tests, incorrectStaffNumber1) {
     musicXmlIoTestRef("testIncorrectStaffNumber1");
 }
 TEST_F(MusicXml_Tests, incorrectStaffNumber2) {
     musicXmlIoTestRef("testIncorrectStaffNumber2");
 }
-TEST_F(MusicXml_Tests, DISABLED_EXCEPT_ON_LINUX(inferredCredits1)) {
+TEST_F(MusicXml_Tests, inferredCredits1) {
     musicXmlImportTestRef("testInferredCredits1");
 }
-TEST_F(MusicXml_Tests, DISABLED_EXCEPT_ON_LINUX(inferredCredits2)) {
+TEST_F(MusicXml_Tests, inferredCredits2) {
     musicXmlImportTestRef("testInferredCredits2");
 }
 TEST_F(MusicXml_Tests, inferCodaII) {
@@ -779,7 +796,7 @@ TEST_F(MusicXml_Tests, inferredDynamicsExpression) {
 TEST_F(MusicXml_Tests, inferredRights) {
     musicXmlImportTestRef("testInferredRights");
 }
-TEST_F(MusicXml_Tests, DISABLED_inferredTechnique) {
+TEST_F(MusicXml_Tests, inferredTechnique) {
     musicXmlImportTestRef("testInferredTechnique");
 }
 TEST_F(MusicXml_Tests, inferredTempoText) {
@@ -806,15 +823,15 @@ TEST_F(MusicXml_Tests, invalidLayout) {
 TEST_F(MusicXml_Tests, invalidTimesig) {
     musicXmlIoTestRef("testInvalidTimesig");
 }
-TEST_F(MusicXml_Tests, invisibleDirection) {
-    musicXmlIoTest("testInvisibleDirection");
-}
 TEST_F(MusicXml_Tests, invisibleElements) {
     musicXmlIoTest("testInvisibleElements");
 }
 TEST_F(MusicXml_Tests, invisibleNote) {
     musicXmlMscxExportTestRef("testInvisibleNote");
 }
+TEST_F(MusicXml_Tests, invisibleTempo) {
+    musicXmlIoTest("testInvisibleTempo");
+}
 TEST_F(MusicXml_Tests, keysig1) {
     musicXmlIoTest("testKeysig1");
 }
@@ -837,7 +854,7 @@ TEST_F(MusicXml_Tests, lines3) {
     musicXmlIoTest("testLines3");
 }
 TEST_F(MusicXml_Tests, lines4) {
-    musicXmlMscxExportTestRef("testLines4");
+    musicXmlIoTest("testLines4");
 }
 TEST_F(MusicXml_Tests, lineDetails) {
     musicXmlIoTest("testLineDetails");
@@ -950,20 +967,20 @@ TEST_F(MusicXml_Tests, nonUniqueThings) {
 TEST_F(MusicXml_Tests, noteAttributes1) {
     musicXmlIoTest("testNoteAttributes1");
 }
-TEST_F(MusicXml_Tests, noteAttributes2) {
+TEST_F(MusicXml_Tests, noteAttributes2Import) {
+    musicXmlImportTestRef("testNoteAttributes2");
+}
+TEST_F(MusicXml_Tests, noteAttributes2io) {
     musicXmlIoTestRef("testNoteAttributes2");
 }
 TEST_F(MusicXml_Tests, noteAttributes3) {
     musicXmlIoTest("testNoteAttributes3");
 }
-TEST_F(MusicXml_Tests, DISABLED_noteAttributes4) {
-    musicXmlImportTestRef("testNoteAttributes2");
-}
 TEST_F(MusicXml_Tests, noteColor) {
     musicXmlIoTest("testNoteColor");
 }
 TEST_F(MusicXml_Tests, noteheadNames) {
-    musicXmlMscxExportTestRef("testNoteheadNames");
+    musicXmlIoTest("testNoteheadNames");
 }
 TEST_F(MusicXml_Tests, noteheadParentheses) {
     musicXmlIoTest("testNoteheadParentheses");
@@ -990,7 +1007,7 @@ TEST_F(MusicXml_Tests, numerals) {
     musicXmlIoTest("testNumerals");
 }
 TEST_F(MusicXml_Tests, ornaments) {
-    musicXmlImportTestRef("testOrnaments");
+    musicXmlIoTest("testOrnaments");
 }
 TEST_F(MusicXml_Tests, overlappingSpanners) {
     musicXmlIoTest("testOverlappingSpanners");
@@ -1070,6 +1087,12 @@ TEST_F(MusicXml_Tests, sound2) {
 TEST_F(MusicXml_Tests, specialCharacters) {
     musicXmlIoTest("testSpecialCharacters");
 }
+TEST_F(MusicXml_Tests, stackedHarm) {
+    bool use302 = MScore::useRead302InTestMode;
+    MScore::useRead302InTestMode = false;
+    musicXmlMscxExportTestRef("testStackedHarm");
+    MScore::useRead302InTestMode = use302;
+}
 TEST_F(MusicXml_Tests, staffEmptiness) {
     musicXmlImportTestRef("testStaffEmptiness");
 }
@@ -1088,11 +1111,14 @@ TEST_F(MusicXml_Tests, stickingLyrics) {
 TEST_F(MusicXml_Tests, stringData) {
     musicXmlIoTest("testStringData");
 }
+TEST_F(MusicXml_Tests, stringMute) {
+    musicXmlImportTestRef("testStringmute");
+}
 TEST_F(MusicXml_Tests, stringVoiceName) {
     musicXmlIoTestRef("testStringVoiceName");
 }
 TEST_F(MusicXml_Tests, swing) {
-    musicXmlMscxExportTestRef("testSwing");
+    musicXmlIoTest("testSwing");
 }
 TEST_F(MusicXml_Tests, systemBrackets1) {
     musicXmlIoTest("testSystemBrackets1");
@@ -1116,7 +1142,11 @@ TEST_F(MusicXml_Tests, DISABLED_EXCEPT_ON_LINUX(systemDistance)) {
     musicXmlMscxExportTestRef("testSystemDistance", true);
 }
 TEST_F(MusicXml_Tests, DISABLED_EXCEPT_ON_LINUX(systemDividers)) {
+#if QT_VERSION >= QT_VERSION_CHECK(6, 8, 0)
+    musicXmlIoTest("testSystemDividers-Qt68", true);
+#else
     musicXmlIoTest("testSystemDividers", true);
+#endif
 }
 TEST_F(MusicXml_Tests, systemObjectStaves) {
     musicXmlImportTestRef("testSystemObjectStaves");
@@ -1136,6 +1166,9 @@ TEST_F(MusicXml_Tests, tablature4) {
 TEST_F(MusicXml_Tests, tablature5) {
     musicXmlIoTestRef("testTablature5");
 }
+TEST_F(MusicXml_Tests, tapping) {
+    musicXmlIoTest("testTapping");
+}
 TEST_F(MusicXml_Tests, tboxAboveBelow1) {
     musicXmlMscxExportTestRef("testTboxAboveBelow1");
 }
@@ -1214,6 +1247,9 @@ TEST_F(MusicXml_Tests, importTie4) {
     // Dolet8 ties to different voices & staves
     musicXmlImportTestRef("importTie4");
 }
+TEST_F(MusicXml_Tests, importTie5) {
+    musicXmlIoTest("importTie5");
+}
 TEST_F(MusicXml_Tests, timesig1) {
     musicXmlIoTest("testTimesig1");
 }
@@ -1223,6 +1259,9 @@ TEST_F(MusicXml_Tests, timesig3) {
 TEST_F(MusicXml_Tests, timesig4) {
     musicXmlIoTest("testTimesig4");
 }
+TEST_F(MusicXml_Tests, timesig5) {
+    musicXmlMscxExportTestRef("testTimesig5");
+}
 TEST_F(MusicXml_Tests, timeTick) {
     musicXmlImportTestRef("testTimeTick");
 }
