@@ -42,7 +42,19 @@ public:
     muse::UriQuery soundsUri() const override;
     muse::UriQuery soundPageUri(const muse::String& soundCode) const override;
 
+    bool needCheckForMuseSoundsUpdate() const override;
+
+    QUrl checkForMuseSoundsUpdateUrl() const override;
+    QUrl checkForMuseSamplerUpdateUrl() const override;
+
+    QString getMuseSamplerVersionQuery() const override;
+
+    std::string lastShownMuseSoundsReleaseVersion() const override;
+    void setLastShownMuseSoundsReleaseVersion(const std::string& version) override;
+
+    bool museSoundsCheckForUpdateTestMode() const override;
+
 private:
-    bool isTestingMode() const;
+    bool getSoundsTestMode() const;
 };
 }
